"""pyUltrasonic Sensor Module."""
import RPi.GPIO as GPIO
import time

class UltrasonicSensorModule():

    def __init__(self, gpio_trigger, gpio_echo, depth, compensation=0):
        """Initialize the sensor object."""
        self.distance = None
        self.startTime = None
        self.distance = None
        self.gpio_trigger = gpio_trigger
        self.gpio_echo = gpio_echo
        self.depth = depth
        self.compensation = compensation

        #GPIO Mode (BOARD / BCM)
        GPIO.setmode(GPIO.BCM)

        #set GPIO direction (IN / OUT)
        GPIO.setup(self.gpio_trigger, GPIO.OUT)
        GPIO.setup(self.gpio_echo, GPIO.IN)

    def __del__(self):
        """clean up all the ports you’ve used."""
        GPIO.cleanup()

    def retrieveDistance(self):
        """Retrieve the distance."""
        # set Trigger to HIGH
        GPIO.output(self.gpio_trigger, True)

        # set Trigger after 0.01ms to LOW
        time.sleep(0.00001)
        GPIO.output(self.gpio_trigger, False)

        self.startTime = time.time()
        startElapsed = time.perf_counter()
        stopElapsed = time.perf_counter()

        # save StartTime
        while GPIO.input(self.gpio_echo) == 0 and (time.perf_counter() - startElapsed) < 30:
            startElapsed = time.perf_counter()
        if (time.perf_counter() - startElapsed) >= 30:
            pass
            #TODO: Raise an Error

        # save time of arrival
        while GPIO.input(self.gpio_echo) == 1 and (time.perf_counter() - stopElapsed) < 30:
            stopElapsed = time.perf_counter()
        if (time.perf_counter() - stopElapsed) >= 30:
            pass
            #TODO: Raise an Error

        # time difference between start and arrival
        disdistance = ((stopElapsed - startElapsed) * 34300) / 2 + self.compensation
        self.distance = round(disdistance, 0)

    def getDistance(self):
        """Return the distance value."""
        if self.distance is None:
            self.retrieveDistance()

        return self.distance

    def waterLevel(self):
        """Return the water level value."""
        if self.distance is None:
            self.retrieveDistance()

        return self.depth - self.distance

    def waterLevelPercent(self):
        """Return the water level in percentage."""
        if self.distance is None:
            self.retrieveDistance()

        return round((self.depth - self.distance) / self.depth * 100 ,0)



"""PyUltrasonic Output Module."""

def output_influx(data):
    """Print data using influxDB format."""

    # Pop yesterdays data
    retrievetime_data = data['retrievetime']
    del data['retrievetime']

    # Print general data
    # out = "pyultrasonic,contract=" + contract + " "
    out = "pyultrasonic,source=\"HC-SR04sensor\" "

    for index, key in enumerate(data):
        if index != 0:
            out = out + ","
        out += key + "=" + str(data[contract][key])

    out += " " + self.ts_utc_from_datestr(last_ts_str) + '000000000'
    print(out)

@staticmethod
def ts_utc_from_datestr(utc_date_str):
    utc = datetime.strptime(utc_date_str, '%Y-%m-%d %H:%M:%S')

    pst = pytz.timezone('Etc/UTC')
    utc = pst.localize(utc)

    # return calendar.timegm(dt.utctimetuple())
    return utc.timestamp()



import argparse

# from pyultrasonic import UltrasonicSensor
from pyultrasonic.output import output_influx

VERSION = "0.1.2"

def main():
    """Entrypoint function."""
    parser = argparse.ArgumentParser()
    parser.add_argument('-t', '--trigger',
                        help='Your GPIO Trigger pin')
    parser.add_argument('-e', '--echo',
                        help='Your GPIO Echo pin')
    parser.add_argument('-d', '--depth',
                        help='Pit depth')
    parser.add_argument('-c', '--compensation',
                        help='Compensation')
    parser.add_argument('-j', '--json', action='store_true',
                        default=False, help='Json output')
    parser.add_argument('-i', '--influxdb', action='store_true',
                        default=False, help='InfluxDb output')

    args = parser.parse_args()

    if args.version:
        print(VERSION)
        return 0

    if not args.trigger or not args.echo:
        parser.print_usage()
        print("pyultrasonic: error: the following arguments are required: "
              "-t/--trigger, -p/--echo, -d/--depth")
        return 3

    sensor = UltrasonicSensorModule(args.trigger, args.echo, args.depth, args.compensation)
    loop = asyncio.get_event_loop()

    sensor.retrieveDistance()

    if args.influxdb:
        output_influx(sensor.get_data())
    elif args.json:
        output_json(client.get_data())
    return 0


if __name__ == '__main__':
    sys.exit(main())





import sys

from setuptools import setup
from pyultrasonic.__main__ import VERSION

if sys.version_info < (3,4):
    sys.exit('Sorry, Python < 3.4 is not supported')

#install_requires = list(val.strip() for val in open('requirements.txt'))
tests_require = list(val.strip() for val in open('test_requirements.txt'))

setup(name='pyultrasonic',
      version=VERSION,
      description='Retrieve distances using a Raspberry Pi with ultrasonic sensor HC-SR04',
      author='Richard Dubois',
      author_email='dubois.richard@gmail.com',
      url='https://github.com/richie256/pyultrasonic',
      package_data={'': ['LICENSE']},
      include_package_data=True,
      packages=['pyultrasonic'],
      entry_points={
          'console_scripts': [
              'pyultrasonic = pyultrasonic.__main__:main'
          ]
      },
      license='Apache 2.0',
      install_requires=['RPi.GPIO'],
      tests_require=tests_require,
      classifiers=[
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
      ]

)
